---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer     
--                                                           
--  RAPID.ADB 
--  Description : Autogeneration widget positions in Dialog
--                       
--  By: Dmitri Anisimkov
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
with Ada.Strings.Fixed,Ada.Strings,Ada.Characters.Handling;
with Tcl.Ada,Interfaces.C,Cargv;
with tcl_utilities;
with tcl_helpers;
with Ada.Unchecked_Deallocation;

package body Auto_Position is

   function Obj_Name(Pos : Positioner;Name : String) return String is
   begin
     return Tcl_Utilities.Window_Object_To_Name(To_String(Pos.Window_Name),Name);
   end;

   procedure Set_Auto_New_Column(Pos : in out Positioner; Row_Count : Integer) is
   begin
     Pos.Row_Auto_New_Column := Row_Count;
   end;

   function Command_Name(Pos : in Positioner; Name : String) return String is
   begin
      return Name&tcl_helpers.Undot_Name(To_String(Pos.Window_Name));
   end;

   procedure Inc_Row_Idx(Pos : in out Positioner) is
   begin
      Pos.Row_Idx := Pos.Row_Idx + 1;
      if Pos.Row_Auto_New_Column=Pos.Row_Idx then
         Start_New_Column(Pos);
      end if;
   end;

   procedure Start_New_Sequence(Pos : in out Positioner;
      Interp : in Tcl.Tcl_Interp;Window_Name : String) is
   begin
      Pos.Max_Row := 0;
      Pos.Col_Idx := 0;
      Pos.Row_Idx := 0;
      Pos.Col_Inc := 0;
      Pos.Interp := Interp;
      Pos.Window_Name := To_Unbounded_String(Window_Name);
      declare
         WName : String := Obj_Name(Pos,"d");
         Last : integer := WName'Last-2;
      begin
         if WName'Length=2 then
           Last := 1;
         else
            Tcl.Ada.Tcl_Eval(Pos.Interp,
            "toplevel " & WName(WName'First..Last) & ASCII.LF);
         end if;
         Tcl.Ada.Tcl_Eval(Pos.Interp,
         "focus " & WName(WName'First..Last) & ASCII.LF);
      end;
   end;

   procedure Check_Max_Row(Pos : in out Positioner) is
   begin
      if Pos.Row_Idx>Pos.Max_Row then
         Pos.Max_Row := Pos.Row_Idx;
      end if;
   end;

   procedure Start_New_Column(Pos : in out Positioner) is
   begin
      Pos.Col_Idx := Pos.Col_Idx+Pos.Col_Inc;
      Check_Max_Row(Pos);
      Pos.Row_Idx := 0;
      Pos.Col_Inc := 0;
   end;

   function Dummy_Str(Pos : Positioner) return String is
      use Ada.Strings;
   begin
      return Fixed.Trim(Integer'Image(Pos.Row_Idx),Left) & '_' &
         Fixed.Trim(Integer'Image(Pos.Col_Idx),Left);
   end;

   function Dummy_Frame(Pos : Positioner) return String is
   begin
      return Obj_Name(Pos,"frame"&Dummy_Str(Pos));
   end;

   function Dummy_Label(Pos : Positioner) return String is
   begin
      return Obj_Name(Pos,Dummy_Str(Pos));
   end;

   procedure Set_Max_Col_Inc(Pos : in out Positioner; Ci : Integer) is
   begin
      if Ci>Pos.Col_Inc then
         Pos.Col_Inc := Ci;
      end if;
   end;

   ---------------
   -- Add_Entry --
   ---------------

   procedure Add_Entry (Pos : in out Positioner;
         Label,Name : String; Value : String:="") is
      Label_Name : constant String := Dummy_Label(Pos);
      OName : String := Obj_Name(Pos,Name);
      Tcl_Input : constant String :=
         "entry "&OName&Ascii.Lf &
         "bind "&OName&" <Key-Return> {"&Command_Name(Pos,"on_ok")&"}"&Ascii.Lf&
         OName & " insert insert {" & Value & '}' & Ascii.Lf &
         "label "& Label_Name &" -text """ & Label & '"' & Ascii.Lf &
         "grid "& Label_Name &" -sticky w" &
         " -column" & Integer'Image(Pos.Col_Idx) &
         " -row"    & Integer'Image(Pos.Row_Idx) & Ascii.Lf &
         "grid "& OName & " -sticky we" &
         " -column" & Integer'Image(Pos.Col_Idx+1) &
         " -row"    & Integer'Image(Pos.Row_Idx) & Ascii.Lf;
   begin
      Tcl.Ada.Tcl_Eval(Pos.Interp,Tcl_Input);
      Inc_Row_Idx(Pos);
      Set_Max_Col_Inc(Pos,2);
   end Add_Entry;

   procedure Add_Entry (Pos : in out Positioner;
         Label,Name : String; Value : Integer) is
   begin
      Add_Entry(Pos,Label,Name,Integer'Image(Value));
   end;

   procedure Add_Message(Pos : in out Positioner; Text, Back_Color : String) is
      Label_Name : constant String := Dummy_Label(Pos);
      Tcl_Input : constant String :=
         "label "&Label_Name&" -text """&Text&""" -bg """&Back_Color&'"'&Ascii.Lf &
         "grid "& Label_Name & " -sticky we -columnspan 2" &
         " -column" & Integer'Image(Pos.Col_Idx) &
         " -row"    & Integer'Image(Pos.Row_Idx) & Ascii.Lf;
   begin
      Tcl.Ada.Tcl_Eval(Pos.Interp,Tcl_Input);
      Inc_Row_Idx(Pos);
      Set_Max_Col_Inc(Pos,2);
   end;

   procedure Add_Check_Button(Pos : in out Positioner;
         Text, Name : String;Value : Boolean := False) is
      OName : String := Obj_Name(Pos,Name);
      Tcl_Input : constant String :=
         "checkbutton "&OName&" -text """&Text&'"'&Ascii.Lf&
         "bind "&OName&" <Key-Return> {"&Command_Name(Pos,"on_ok")&"}"&Ascii.Lf&
         "grid "&OName&" -sticky w -columnspan 2"&
         " -column" & Integer'Image(Pos.Col_Idx) &
         " -row"    & Integer'Image(Pos.Row_Idx) & Ascii.Lf;
   begin
      Tcl.Ada.Tcl_Eval(Pos.Interp,Tcl_Input);
      if Value then
         Tcl.Ada.Tcl_Eval(Pos.Interp,""&OName&" select"&Ascii.Lf);
      end if;
      Inc_Row_Idx(Pos);
      Set_Max_Col_Inc(Pos,2);
   end;

   package body enumeration_entry is

   function To_String(e : enum) return string is
   begin
      return Ada.Characters.Handling.To_Lower(Enum'Image(e));
   end;

   function Get_Enum_Entry(Interp : Tcl.Tcl_Interp;
      Window_Name : String; Name : String) return string is
     Word : String(1..64);
     Last : Integer;
   begin
      Tcl_Utilities.Get_Value(Interp,
         Name        =>
      tcl_helpers.Undot_Name(
         Tcl_Utilities.Window_Object_To_Name(
            Window_Name,Name)),
         Result      => Word,
         Last        => Last);
      return  Word(Word'First..Last);
   end;

   function Get_Enum_Entry(Interp : Tcl.Tcl_Interp;
      Window_Name : String; Name : String) return Enum is
   begin
      return Enum'Value(Get_Enum_Entry(Interp,Window_Name,Name));
   end;

   procedure Add_Entry(Pos : in out Positioner;
         Label,Name : String;Value : String) is
      Label_Name : constant String := Dummy_Label(Pos);
      function Enum_Strings return String is
         Cnt : Integer := 0;
         Parts : array(Enum) of Integer;
      begin
         for I in Enum'range loop
            Cnt := Cnt+Enum'Image(I)'Length+1;
            Parts(I) := Cnt;
         end loop;
         declare
            Result : String(1..Cnt);
         begin
            Cnt := 1;
            for I in Enum'range loop
               Result(Cnt..Parts(I)) := ' '&To_String(I);
               Cnt := Parts(I)+1;
            end loop;
            return Result;
         end;
      end;
      OName : String := Obj_Name(Pos,Name);
      Entry_Name : constant string := tcl_helpers.Undot_Name(OName);
      Tcl_Input : constant String :=
         "tk_optionMenu "&OName&' '&Entry_Name&' '&Enum_Strings&Ascii.Lf&
         "bind "&OName&" <Key-Return> {"&Command_Name(Pos,"on_ok")&"}"&Ascii.Lf&
         "label "& Label_Name &" -text """ & Label & '"' & Ascii.Lf &
         "grid "& Label_Name &" -sticky w" &
         " -column" & Integer'Image(Pos.Col_Idx) &
         " -row"    & Integer'Image(Pos.Row_Idx) & Ascii.Lf &
         "grid "&OName & " -sticky we" &
         " -column" & Integer'Image(Pos.Col_Idx+1) &
         " -row"    & Integer'Image(Pos.Row_Idx) & Ascii.Lf;
   begin
      Tcl.Ada.Tcl_Eval(Pos.Interp,Tcl_Input);
      if Value/="" then
         Tcl.Ada.Tcl_Eval(Pos.Interp,"set "&Entry_Name&" """&Value&'"'&Ascii.Lf);
      end if;
      Inc_Row_Idx(Pos);
      Set_Max_Col_Inc(Pos,2);
   end;

   procedure Add_Entry(Pos : in out Positioner;
      Label,Name : String; Value : Enum := Enum'First) is
   begin
      Add_Entry(Pos,Label,Name,To_String(Value));
   end;

   end;

   type Internal_Params is record
      On_Apply,On_Ok,On_Cancel : Dialog_Button_Proc;
      Window_Name : Unbounded_String;
   end record;

   type Internal_Ptr is access all Internal_Params;

   procedure Free_Params is new Ada.Unchecked_Deallocation(Internal_Params,Internal_Ptr);

   package Create_Commands is new Tcl.Ada.Generic_Command(Internal_Ptr);

   function On_Apply_Internal (
      Clientdata : in Internal_Ptr;
      Interp     : in Tcl.Tcl_Interp;
      Argc       : in Interfaces.C.Int;
      Argv       : in Cargv.Chars_Ptr_Ptr
      ) return Interfaces.C.Int;
   pragma Convention (C, On_Apply_Internal);

   function On_Apply_Internal (
         Clientdata : in Internal_Ptr;
         Interp     : in Tcl.Tcl_Interp;
         Argc       : in Interfaces.C.Int;
         Argv       : in Cargv.Chars_Ptr_Ptr
         ) return Interfaces.C.Int is
   begin
      if Clientdata.On_Apply/=null then
         Clientdata.On_Apply.all;
      end if;
      return Tcl.Tcl_Ok;
   end On_Apply_Internal;

   function On_Ok_Internal(
      Clientdata : in Internal_Ptr;
      Interp     : in Tcl.Tcl_Interp;
      Argc       : in Interfaces.C.Int;
      Argv       : in Cargv.Chars_Ptr_Ptr
      ) return Interfaces.C.Int;
   pragma Convention (C, On_Ok_Internal);

   function On_Ok_Internal(
         Clientdata : in Internal_Ptr;
         Interp     : in Tcl.Tcl_Interp;
         Argc       : in Interfaces.C.Int;
         Argv       : in Cargv.Chars_Ptr_Ptr
         ) return Interfaces.C.Int is
   begin
      if Clientdata.On_Ok/=null then
         Clientdata.On_Ok.all;
      end if;
      return Tcl.Tcl_Ok;
   end On_Ok_Internal;

   function On_Cancel_Internal (
      Clientdata : in Internal_Ptr;
      Interp     : in Tcl.Tcl_Interp;
      Argc       : in Interfaces.C.Int;
      Argv       : in Cargv.Chars_Ptr_Ptr
      ) return Interfaces.C.Int;
   pragma Convention (C, On_Cancel_Internal);

   function On_Cancel_Internal (
         Clientdata : in Internal_Ptr;
         Interp     : in Tcl.Tcl_Interp;
         Argc       : in Interfaces.C.Int;
         Argv       : in Cargv.Chars_Ptr_Ptr
         ) return Interfaces.C.Int is
   begin
      if Clientdata.On_Cancel/=null then
         Clientdata.On_Cancel.all;
      end if;
--      Tcl_Utilities.Destroy_Window(Interp,To_String(ClientData.Window_Name));
      return Tcl.Tcl_Ok;
   end On_Cancel_Internal;

   function On_Destroy_Internal (
      Clientdata : in Internal_Ptr;
      Interp     : in Tcl.Tcl_Interp;
      Argc       : in Interfaces.C.Int;
      Argv       : in Cargv.Chars_Ptr_Ptr
      ) return Interfaces.C.Int;
   pragma Convention (C, On_Destroy_Internal);

   function On_Destroy_Internal (
         Clientdata : in Internal_Ptr;
         Interp     : in Tcl.Tcl_Interp;
         Argc       : in Interfaces.C.Int;
         Argv       : in Cargv.Chars_Ptr_Ptr
         ) return Interfaces.C.Int is
         cd : Internal_Ptr := ClientData;
   begin
      Free_Params(CD);
      return Tcl.Tcl_Ok;
   end On_Destroy_Internal;

   procedure Add_Apply_Ok_Cancel(Pos : in out Positioner;
      On_Apply : Dialog_Button_Proc;
      On_Ok : Dialog_Button_Proc := null;
      On_Cancel : Dialog_Button_Proc := null) is
      Apply_Name : String := Obj_Name(Pos,"applyButton");
      Ok_Name : String := Obj_Name(Pos,"okButton");
      Cancel_Name : String := Obj_Name(Pos,"cancelButton");
      On_Apply_Name : constant string := Command_Name(Pos,"on_apply");
      On_Ok_Name : constant string := Command_Name(Pos,"on_ok");
      On_Cancel_Name : constant string := Command_Name(Pos,"on_cancel");
      On_Destroy_Name : constant string := Command_Name(Pos,"on_destroy");
      ClientData : Internal_Ptr := new Internal_Params'(On_Apply,On_Ok,On_Cancel,Pos.Window_Name);
   begin
      Check_Max_Row(Pos);
      declare
         Frame_Name : constant String := Dummy_Frame(Pos);
         Tcl_Input : constant String :=
            "frame "&Frame_Name& Ascii.Lf &
            "button "&Apply_Name&" -text ""Apply"" -command { "&on_apply_name&" }" & Ascii.Lf &
            "button "&Ok_Name&" -text ""OK"" -command { "&on_ok_name&" }" & Ascii.Lf &
            "button "&Cancel_Name&" -text ""Cancel"" -command { "&on_cancel_name&" }" & Ascii.Lf &
            "grid "&Frame_Name&" -sticky we -columnspan 2" &
            " -column" & Integer'Image(Pos.Col_Idx) &
            " -row"    & Integer'Image(Pos.Max_Row) & Ascii.Lf&
            "pack "&Apply_Name&' '&Ok_Name&' '&Cancel_Name&' '&
            "-side left -fill x -expand 1 -in "&Frame_Name& Ascii.Lf;
         Command : Tcl.Tcl_Command;
      begin
         Tcl.Ada.Tcl_Eval(Pos.Interp,Tcl_Input);
         if ClientData.On_Ok = null then
            ClientData.On_Ok := On_Apply;
         end if;
         Command := Create_Commands.Tcl_Createcommand (Pos.Interp,
            on_apply_name, On_Apply_Internal'access, ClientData, null);
         Command := Create_Commands.Tcl_Createcommand (Pos.Interp,
            on_ok_name, On_Ok_Internal'access, ClientData, null);
         Command := Create_Commands.Tcl_Createcommand (Pos.Interp,
            on_cancel_name, On_Cancel_Internal'access, ClientData, null);
         Command := Create_Commands.Tcl_Createcommand (Pos.Interp,
            on_destroy_name, On_Destroy_Internal'access, ClientData, null);
         Tcl.Ada.Tcl_Eval(Pos.Interp,"bind "&Ok_Name&" <Destroy> "&
            on_destroy_name);
      end;
   end Add_Apply_Ok_Cancel;

end Auto_Position;