---------------------------------------------------------------
--                                                           
--  B64_TO_TASH     
--  Description : Converts Base-64 encoded GIFs to TASH files
--                       
--  By: Martin Carlisle
--                                         
-- B64_TO_TASH is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- B64_TO_TASH is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
-- This program takes a GIF file that has been encoded in Base-64
-- Mypicture.B64 (for example) and creates TASH files that will
-- load the picture (mypicture.ad[bs]).
--
-- Usage: b64_to_tash mypicture.b64 mypicture
-- The Ada package name (given second) does not have to match
-- the name of the base 64 file.
---------------------------------------------------------------
with ada.text_io;
with ada.command_line;
with b64_to_tash_utils;

procedure b64_to_tash is
   procedure usage is
   begin
      Ada.Text_IO.Put_Line("usage: " & Ada.Command_Line.Command_Name &
         " input_file Ada_package_name");
   end usage;

begin
   -- requires 2 arguments.  If not, print usage message and exit.
   if Ada.Command_Line.Argument_Count /= 2 then
      usage;
      return;
   end if;

   -- call helper package to do all the work
   b64_to_tash_utils(Ada.Command_Line.Argument(1),
      Ada.Command_Line.Argument(2));

exception
   -- if helper package generates exception opening a file, print error
   -- here.
   when Ada.Text_IO.Name_Error =>
      Ada.Text_IO.Put_Line("Invalid filename specified");
end b64_to_tash;