---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer
--                                                           
--  GUI-MENU.ADB 
--  Description : Root of GUI MENU Hierarchy
--                       
--  By: Martin Carlisle
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
package body Gui.Menu is

   --------------------------------------------------
   -- procedure Renumber_Menus
   --
   -- after an insertion, renumber all menus
   --
   -- This guarantees menus are numbered 1,2,3... in
   -- order.
   --
   -- This is pretty bad (O(n^2) for n insertions), but
   -- speed isn't an issue.
   --
   -- returns count of things in the menu
   -- Algorithm
   -- 1) Simply call helper starting at 1
   -- 2) Count from helper is one too big, so decrement
   --------------------------------------------------
   procedure Renumber_Menus(Menubar : in Menu_Pointer;
      Menu_Count : out Integer) is
      -----------------------------------------------
      -- subprocedure Renumber_Menus_Helper
      --
      -- Allows us to do this recursively, by making 
      -- Count be in/out.
      -- 
      -- Since it post increments, answer will be one
      -- too big.
      -----------------------------------------------
      procedure Renumber_Menus_Helper(Menubar : in Menu_Pointer;
         Count : in out Integer) is
         Current_Position: Gui.Menu.Menu_Position :=
            Gui.Menu.Menu_List_Package.First(Menubar);
         Current_Menu : Gui.Menu.Menu_Access;
      begin
         -- Loop through the list of menu objects
         -- Set current # to count and increment
         -- if it's a submenu, make a recursive call
         while not Gui.Menu.Menu_List_Package.IsPastEnd(
               Menubar,Current_Position) loop
            Current_Menu := Gui.Menu.Menu_List_Package.Retrieve(Menubar,
               Current_Position);
            Current_Menu.Number := Count;
            Count := Count + 1;
            if Current_Menu.all in GUI.Menu.Submenu'Class then
               Renumber_Menus_Helper(
                  Gui.Menu.Submenu(Current_Menu.all).Items,
                  Count);
            end if;
            Gui.Menu.Menu_List_Package.GoAhead(
               Menubar,Current_Position);
         end loop;
      end Renumber_Menus_Helper;

      Count    : Integer := 1;
   begin -- Renumber_Menus
      Renumber_Menus_Helper(Menubar,Count);
      Menu_Count := Count - 1; -- we post-increment, so answer is -1
   end Renumber_Menus;

end Gui.Menu;