---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer
--                                                           
--  GUI_MENU.ADS 
--  Description : Implements Menus/Choices
--                       
--  By: Martin Carlisle
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
-- Contains data types for menus.
--
-- Menu_Access can point to any kind of menu (just one)
--
-- Menu_Pointer points to a list of menus using Feldman's
-- Lists_Generic package
--
-- Menu Hierarchy
--
--             Menu
--            /    \
--      Menu_Item  Submenu
--
-- Submenus are lists of menu items and submenus 
-- (thus contain a Menu_Pointer)
---------------------------------------------------------------
with Lists_Generic;

package Gui.Menu is

   type Menu;
   type Menu_Access is access all Menu'Class;

   package Menu_List_Package is new Lists_Generic(ElementType => Menu_Access);
   subtype Menu_Pointer is Menu_List_Package.List;
   subtype Menu_Position is Menu_List_Package.Position;

   --------------------------------------------------
   -- procedure Renumber_Menus
   --
   -- after an insertion, renumber all menus
   --
   -- This guarantees menus are numbered 1,2,3... in
   -- order.
   --
   -- returns count of things in the menu
   --------------------------------------------------
   procedure Renumber_Menus(Menubar : in Menu_Pointer;
      Menu_Count : out Integer);

   -- all menus can have a name (how displayed)
   -- underline position (accelerator key is Alt+Underlined)
   -- Action (Ada procedure called if this is selected)
   -- Number (used only in generation phase)
   -- Indent_Level (how nested is this?)
   type Menu is tagged limited record
      Name                 : String_Pointer;
      Underline_Position   : Natural; -- zero based
      Action               : String_Pointer;
      Number               : Natural;
      Indent_Level         : Natural; -- used by menu edit
   end record;

   -- Menu_Items can display an accelerator (e.g. Ctrl+Z)
   type Menu_Item is new Menu with record
      Accelerator          : String_Pointer;
   end record;
   type Menu_Item_Pointer is access all Menu_Item'Class;

   -- A submenu displays a nested menu (list of items, submenus)
   type Submenu is new Menu with record
      Items                : Menu_Pointer;
      Insert_Inside        : Boolean := True; 
      -- used by display, do we insert after the submenu, or inside it?
      -- is toggled in menu editing.
   end record;
   type Submenu_Pointer is access all Submenu'Class;
end Gui.Menu;