---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer
--                                                           
--  GUI-WIDGET-BUTTON.ADB 
--  Description : Root of GUI Widget Button Hierarchy
--                       
--  By: Martin Carlisle
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
-- Change log:
-- 08/21/98 (mcc) : added test to remove ".gif" if specified
-- 02/14/99 Dmitri Anisimkov : Set_Property_Entries instead Set_Properties
-- for creation inheritable propery editors without designer
---------------------------------------------------------------
with Ada.Text_IO;
with File_Helpers;
with Tcl_Helpers,Tcl_Utilities;
with Gui_Enum;
with Tcl.Ada;
with Main;
with Gif_To_Tash;
with Ada.Characters.Handling;
with Auto_Position;

-- debugging
with Common_dialogs;

package body Gui.Widget.Button is
   -- reads information from file into GUI_Widget,
   -- assumes keyword already read.

   procedure Get_Widget_Params(Widget : in out Button;
      Params : in String) is
   begin
      Get_Widget_Params(Gui_Widget(Widget),Params);
      Widget.Action := new String'(Get_Widget_Param(Params,"action"));
   end;

   procedure Read_Widget(Widget : in out Button;
         File        : in Ada.Text_IO.File_Type) is
      Word : Word_Type;
      Last : Natural;
   begin -- Read_Widget
      Read_Widget(Gui_Widget(Widget),File);

      File_Helpers.Get_String(File,Word,Last);
      Widget.Action := new String'(Word(1..Last));
   end Read_Widget;

   -- Writes information to file from GUI_Widget

   procedure Write_Widget(Widget : in Button;
         File : in Ada.Text_IO.File_Type) is

   begin -- Write_Widget
      Write_Widget(Gui_Widget(Widget),File);
      Put_Widget_Param(File,"action",Widget.Action.all);
   end Write_Widget;

   -- Generate Ada/Tcl code for this window.

   procedure Generate_Widget(Widget : in Button;
         Window_Name : in String;
         File : in Ada.Text_IO.File_Type;
         Tcl_Only : in Boolean) is

   begin -- Generate_Widget
      Generate_Widget(Gui_Widget(Widget),Window_Name,File,Tcl_Only);
   end Generate_Widget;

   procedure Generate_Action(Widget : in Button;
      File : in Ada.Text_IO.File_Type) is
   begin
      Tcl_Helpers.Generate_Command(File => File,
         Action => Widget.Action.all,
         Command_Name => "button" &
            Tcl_Helpers.Undot_Name(Widget.Name.all));
   end Generate_Action;

   procedure Generate_Action_Binding(Widget : in Button;
      File : in Ada.Text_IO.File_Type) is
   begin
      Tcl_Helpers.Generate_Command_Binding(File => File,
         Command_Name => "button" &
            Tcl_Helpers.Undot_Name(Widget.Name.all));
   end Generate_Action_Binding;

   procedure Generate_Context_Clause(Widget : in Button;
      File : in Ada.Text_IO.File_Type) is
   begin
      Tcl_Helpers.Generate_With(File,Widget.Action.all);
   end Generate_Context_Clause;

   procedure Set_Property_Entries(Widget : in out Button;
      Pos : in out Auto_Position.Positioner) is
   begin
      Set_Property_Entries(Gui_Widget(Widget),Pos);
      Auto_Position.Add_Message(Pos,"Button properties","LightBlue");
      Auto_Position.Add_Entry(Pos,
            Label => "Action",
            Name     => "entry6",
            Value     => Tcl_Utilities.Unescape(Unaccess(Widget.Action)));
   end Set_Property_Entries;

   procedure Apply_Properties(Widget : in out Button) is
      Word        : Word_Type;
      Last        : Integer;
      Window_Name : String := Properties_Dialog_Name(Button'Class(Widget));
   begin
      Apply_Properties(Gui_Widget(Widget));
      Tcl_Utilities.Get_Text_Entry(Main.Interp,
         Window_Name => Window_Name,
         Name        => "entry6",
         Result      => Word,
         Last        => Last);
      if Last >= Word'First then
         Widget.Action := new String'(Word(Word'First..Last));
      end if;
   end Apply_Properties;

   procedure Check_Properties(Widget : in out Button;
      Ok : out Boolean) is
      Window_Name : String := Properties_Dialog_Name(
         Gui_Widget'Class(Widget));
   begin
      Check_Properties(GUI_Widget(Widget),Ok);
      if Ok then
         Ok := Widget.Action /= null;
         if not Ok then
            Tcl_Utilities.Highlight_Text_Entry(Main.Interp,
               Window_Name,"entry6");
         end if;
      end if;

      if not Ok then
         Tcl.Ada.Tcl_Eval(Main.Interp,"bell ");
      end if;
   end Check_Properties;

   procedure Get_Widget_Params(Widget : in out Text_Button;
      Params : in String) is
   begin
      Get_Widget_Params(Button(Widget),Params);
      Widget.Text := new String'(Get_Widget_Param(Params,"text","undefined"));
   end;

   -- reads information from file into GUI_Widget,
   -- assumes keyword already read.
   procedure Read_Widget(Widget : in out Text_Button;
         File        : in Ada.Text_IO.File_Type) is
      Word : Word_Type;
      Last : Natural;
   begin -- Read_Widget
      Read_Widget(Button(Widget),File);
      File_Helpers.Get_String(File,Word,Last);
      Widget.Text := new String'(Word(1..Last));
   end Read_Widget;

   -- Writes information to file from GUI_Widget

   procedure Write_Widget(Widget : in Text_Button;
         File : in Ada.Text_IO.File_Type) is

   begin -- Write_Widget
      Gui_Enum.IO.Put(File,Gui_Enum.TextButton);
      Write_Widget(Button(Widget),File);
      Put_Widget_Param(File,"text",Widget.Text.all);
      Ada.Text_IO.New_Line(File);
   end Write_Widget;

   -- Generate Ada/Tcl code for this window.

   procedure Generate_Widget(Widget : in Text_Button;
         Window_Name : in String;
         File : in Ada.Text_IO.File_Type;
         Tcl_Only : in Boolean) is

   begin -- Generate_Widget
      Tcl_Helpers.Begin_Block(File,Tcl_Only);
      Tcl_Helpers.Put(File,
         "button " &
         Window_Name & Widget.Name.all &
         " -text ",Tcl_Only);
      Tcl_Helpers.Put_Quotation(File,Tcl_Only);
      Tcl_Helpers.Put(File,
         Widget.Text.all,Tcl_Only);
      Tcl_Helpers.Put_Quotation(File,Tcl_Only);
      Tcl_Helpers.Put_Line(File," -command { " &
         "button" & Tcl_Helpers.Undot_Name(Widget.Name.all) & " }",Tcl_Only);
      Generate_Widget(Button(Widget),
         Window_Name,File,Tcl_Only);
      Tcl_Helpers.End_Block(File,Tcl_Only);
   end Generate_Widget;

   -- display the widget to a window
   procedure Display_Widget(Widget : in Text_Button;
      Interp      : in Tcl.Tcl_Interp;
      Window_Name : in String) is
   begin
      Tcl.Ada.Tcl_Eval(Interp,
         "button " &
         Window_Name & "." & Widget.Name.all &
         " -text """ &
         Widget.Text.all & '"');
      Display_Widget(Gui_Widget(Widget),Interp,Window_Name);
   exception when others =>
      Common_Dialogs.OK_Box("Can't display: " &
         Widget.Name.all, Interp);
   end Display_Widget;

   procedure Set_Property_Entries(Widget : in out Text_Button;
      Pos : in out Auto_Position.Positioner) is
   begin
      Set_Property_Entries(Button(Widget),pos);
      Auto_Position.Add_Entry(pos,
            Label => "Text",
            Name   => "entry7",
            Value    => Tcl_Utilities.Unescape(Unaccess(Widget.Text)));
   end Set_Property_Entries;

   procedure Apply_Properties(Widget : in out Text_Button) is
      Word        : Word_Type;
      Last        : Integer;
      Window_Name : String := Properties_Dialog_Name(Widget);
   begin
      Apply_Properties(Button(Widget));
      Tcl_Utilities.Get_Text_Entry(Main.Interp,
         Window_Name => Window_Name,
         Name        => "entry7",
         Result      => Word,
         Last        => Last);
      if Last >= Word'First then
         Widget.Text := new String'(Tcl_Utilities.Fix_Quotes(
            Word(Word'First..Last)));
      end if;
   end Apply_Properties;

   procedure Check_Properties(Widget : in out Text_Button;
      Ok : out Boolean) is
      Window_Name : String := Properties_Dialog_Name(Widget);
   begin
      Check_Properties(Button(Widget),Ok);
      if Ok then
         Ok := Widget.Text /= null;
         if not Ok then
            Tcl_Utilities.Highlight_Text_Entry(Main.Interp,
               Window_Name,"entry7");
         end if;
      end if;

      if not Ok then
         Tcl.Ada.Tcl_Eval(Main.Interp,"bell ");
      end if;
   end Check_Properties;


   -- return name of properties dialog
   function Properties_Dialog_Name(Widget : in Text_Button) return String is
   begin
      return ".textbutton_dialog";
   end Properties_Dialog_Name;

   procedure Get_Widget_Params(Widget : in out Picture_Button;
      Params : in String) is
   begin
      Get_Widget_Params(Button(Widget),Params);
      Widget.picture := new String'(Get_Widget_Param(Params,"image"));
   end;

   -- reads information from file into GUI_Widget,
   -- assumes keyword already read.

   procedure Read_Widget(Widget : in out Picture_Button;
         File        : in Ada.Text_IO.File_Type) is
      Word : Word_Type;
      Last : Natural;
   begin -- Read_Widget
      Read_Widget(Button(Widget),File);
      File_Helpers.Get_String(File,Word,Last);
      Widget.Picture := new String'(Word(1..Last));
   end Read_Widget;

   -- Writes information to file from GUI_Widget

   procedure Write_Widget(Widget : in Picture_Button;
         File : in Ada.Text_IO.File_Type) is

   begin -- Write_Widget
-- And now we can't inherit from this widget. Too bad.
      Gui_Enum.IO.Put(File,Gui_Enum.PictureButton);
      Write_Widget(Button(Widget),File);
      Put_Widget_Param(File,"image", Widget.Picture.all);
      Ada.Text_IO.New_Line(File);
   end Write_Widget;

   -- Generate Ada/Tcl code for this window.

   procedure Generate_Widget(Widget : in Picture_Button;
         Window_Name : in String;
         File : in Ada.Text_IO.File_Type;
         Tcl_Only : in Boolean) is

   begin -- Generate_Widget
      if Tcl_Utilities.Is_Newer(Main.Interp,Widget.Picture.all & ".gif",
            Widget.Picture.all & ".adb") then
         begin
            Gif_To_Tash.Convert(Widget.Picture.all);
         exception
            when others =>
               Common_Dialogs.OK_Box("Unable to compile: " & ASCII.LF &
                  Widget.Picture.all & ".gif",Main.Interp);
         end;
      end if;

      Tcl_Helpers.Begin_Block(File,Tcl_Only);
      Tcl_Helpers.Put(File,
         "button " &
         Window_Name & Widget.Name.all &
         " -image ",Tcl_Only);
      Tcl_Helpers.Put_Quotation(File,Tcl_Only);
      Tcl_Helpers.Put(File,
         Widget.Picture.all,Tcl_Only);
      Tcl_Helpers.Put_Quotation(File,Tcl_Only);
      Tcl_Helpers.Put_Line(File," -command { " &
         "button" & Tcl_Helpers.Undot_Name(Widget.Name.all) & " }",Tcl_Only);
      Generate_Widget(Button(Widget),
         Window_Name,File,Tcl_Only);
      Tcl_Helpers.End_Block(File,Tcl_Only);
   end Generate_Widget;

   procedure Generate_Context_Clause(Widget : in Picture_Button;
      File : in Ada.Text_IO.File_Type) is
   begin
      Generate_Context_Clause(Button(Widget),File);
      Tcl_Helpers.Generate_With(File,Widget.Picture.all & ".");
   end Generate_Context_Clause;

   procedure Generate_Action_Binding(Widget : in Picture_Button;
      File : in Ada.Text_IO.File_Type) is
   begin
      Generate_Action_Binding(Button(Widget),File);
      Ada.Text_IO.Put_Line(File,
         "      " & Widget.Picture.all & "." &
         "Generate_Image(Interp);");
   end Generate_Action_Binding;

   -- display the widget to a window
   procedure Display_Widget(Widget : in Picture_Button;
      Interp      : in Tcl.Tcl_Interp;
      Window_Name : in String) is
      Excepted : Boolean := False;
   begin
      begin
         Tcl.Ada.Tcl_Eval(Interp,
            "image create photo " &
            Widget.Picture.all &
            " -file " &
            Widget.Picture.all & ".gif");
      exception
         when others =>
            if Widget.Picture /= null then
               Common_Dialogs.OK_Box("Unable to find picture: " &
                  Widget.Picture.all, Interp);
            end if;
            Tcl.Ada.Tcl_Eval(Interp,
               "button " &
               Window_Name & "." & Widget.Name.all);
            Excepted := True;
      end;

      if not Excepted then
         Tcl.Ada.Tcl_Eval(Interp,
            "button " &
            Window_Name & "." & Widget.Name.all &
            " -image """ &
            Widget.Picture.all & '"');
      end if;

      Display_Widget(Gui_Widget(Widget),Interp,Window_Name);
   exception when others =>
      Common_Dialogs.OK_Box("Can't display: " &
         Widget.Name.all, Interp);
   end Display_Widget;

   procedure Set_Property_Entries(Widget : in out Picture_Button;
      Pos : in out Auto_Position.Positioner) is
   begin
      Set_Property_Entries(Button(Widget),pos);
      Auto_Position.Add_Entry(pos,
            Label => "Picture",
            Name    => "entry7",
            Value    => Unaccess(Widget.Picture));
   end Set_Property_Entries;

   procedure Apply_Properties(Widget : in out Picture_Button) is
      Word        : Word_Type;
      Last        : Integer;
      Window_Name : String := Properties_Dialog_Name(Widget);
   begin
      Apply_Properties(Button(Widget));
      Tcl_Utilities.Get_Text_Entry(Main.Interp,
         Window_Name => Window_Name,
         Name        => "entry7",
         Result      => Word,
         Last        => Last);
      if Last >= Word'First then
         -- remove ".gif" if present
         if Last > Word'First+3 and then
            Ada.Characters.Handling.To_Lower(Word(Last-3..Last)) = ".gif"
            then
            Last := Last - 4;
         end if;
         Widget.Picture := new String'(Word(Word'First..Last));
      end if;
   end Apply_Properties;

   procedure Check_Properties(Widget : in out Picture_Button;
      Ok : out Boolean) is
      Window_Name : String := Properties_Dialog_Name(Widget);
   begin
      Check_Properties(Button(Widget),Ok);
      if Ok then
         Ok := Widget.Picture /= null;
         if not Ok then
            Tcl_Utilities.Highlight_Text_Entry(Main.Interp,
               Window_Name,"entry7");
         end if;
      end if;

      if not Ok then
         Tcl.Ada.Tcl_Eval(Main.Interp,"bell ");
      end if;
   end Check_Properties;


   -- return name of properties dialog
   function Properties_Dialog_Name(Widget : in Picture_Button) return String is
   begin
      return ".picturebutton_dialog";
   end Properties_Dialog_Name;


end Gui.Widget.Button;