---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer
--                                                           
--  GUI-WIDGET-BUTTON.ADS 
--  Description : Root of GUI Widget Button Hierarchy
--                       
--  By: Martin Carlisle
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
-- Additions to hierarchy
--               GUI_Widget
--                   |
--                 Button
--                /      \
--         Text_Button  Picture_Button
--
-- Button adds action (for press)
-- Text_Button adds Text (label)
-- Picture_Button adds Picture -- name of GIF (without .GIF extension)
---------------------------
-- 02/14/99 Dmitri Anisimkov : Set_Property_Entries instead Set_Properties
-- for creation inheritable propery editors without designer
---------------------------------------------------------------
with ada.text_io;
with Auto_Position;

package Gui.Widget.Button is
   type Button is abstract new Gui.Widget.GUI_Widget with record
      Action           : String_Pointer;
   end record;

   -- reads information from file into GUI_Widget,
   -- assumes keyword already read.
   procedure Read_Widget(Widget : in out Button;
      File        : in Ada.Text_IO.File_Type);

   procedure Get_Widget_Params(Widget : in out Button; 
      Params : in String);

   -- Writes information to file from GUI_Widget
   procedure Write_Widget(Widget : in Button;
      File : in Ada.Text_IO.File_Type);

   -- Generate Ada/Tcl code for this window.
   procedure Generate_Widget(Widget : in Button;
      Window_Name : in String;
      File : in Ada.Text_IO.File_Type;
      Tcl_Only : in Boolean);

   procedure Generate_Action(Widget : in Button;
      File : in Ada.Text_IO.File_Type);

   procedure Generate_Action_Binding(Widget : in Button;
      File : in Ada.Text_IO.File_Type);

   procedure Generate_Context_Clause(Widget : in Button;
      File : in Ada.Text_IO.File_Type);

   procedure Set_Property_Entries(Widget : in out Button;
      Pos : in out Auto_Position.Positioner);

   procedure Apply_Properties(Widget : in out Button);

   procedure Check_Properties(Widget : in out Button;
      Ok : out Boolean);

   type Text_Button is new Button with record
      Text : String_Pointer;
   end record;

   procedure Get_Widget_Params(Widget : in out Text_Button; 
      Params : in String);
      
   -- reads information from file into GUI_Widget,
   -- assumes keyword already read.
   procedure Read_Widget(Widget : in out Text_Button;
      File        : in Ada.Text_IO.File_Type);

   -- Writes information to file from GUI_Widget
   procedure Write_Widget(Widget : in Text_Button;
      File : in Ada.Text_IO.File_Type);

   -- Generate Ada/Tcl code for this window.
   procedure Generate_Widget(Widget : in Text_Button;
      Window_Name : in String;
      File : in Ada.Text_IO.File_Type;
      Tcl_Only : in Boolean);

   -- display the widget to a window
   procedure Display_Widget(Widget : in Text_Button;
      Interp      : in Tcl.Tcl_Interp;
      Window_Name : in String);

   procedure Set_Property_Entries(Widget : in out Text_Button;
      Pos : in out Auto_Position.Positioner);

   procedure Apply_Properties(Widget : in out Text_Button);
   procedure Check_Properties(Widget : in out Text_Button;
      Ok : out Boolean);

   function Properties_Dialog_Name(Widget : in Text_Button) return String;

   type Picture_Button is new Button with record
      Picture : String_Pointer;
   end record;

   procedure Get_Widget_Params(Widget : in out Picture_Button; 
      Params : in String);
      
   -- reads information from file into GUI_Widget,
   -- assumes keyword already read.
   procedure Read_Widget(Widget : in out Picture_Button;
      File        : in Ada.Text_IO.File_Type);

   -- Writes information to file from GUI_Widget
   procedure Write_Widget(Widget : in Picture_Button;
      File : in Ada.Text_IO.File_Type);

   -- Generate Ada/Tcl code for this window.
   procedure Generate_Widget(Widget : in Picture_Button;
      Window_Name : in String;
      File : in Ada.Text_IO.File_Type;
      Tcl_Only : in Boolean);

   procedure Generate_Context_Clause(Widget : in Picture_Button;
      File : in Ada.Text_IO.File_Type);

   procedure Generate_Action_Binding(Widget : in Picture_Button;
      File : in Ada.Text_IO.File_Type);

   -- display the widget to a window
   procedure Display_Widget(Widget : in Picture_Button;
      Interp      : in Tcl.Tcl_Interp;
      Window_Name : in String);

   procedure Set_Property_Entries(Widget : in out Picture_Button;
      Pos : in out Auto_Position.Positioner);

   procedure Apply_Properties(Widget : in out Picture_Button);
   procedure Check_Properties(Widget : in out Picture_Button;
      Ok : out Boolean);

   function Properties_Dialog_Name(Widget : in Picture_Button) return String;
end Gui.Widget.Button;