---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer
--                                                           
--  GUI-WIDGET-LISTBOX.ADS
--  Description : GUI Widget Listbox
--                       
--  By: Jonathan Busch
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
with Tcl_Helpers;
with Auto_Position;

package Gui.Widget.Listbox is

   type Listbox is new GUI_Widget with record
      FG_Color	      : String_Pointer := NULL;
      BG_Color	      : String_Pointer := NULL;
      Select_Color    : String_Pointer := NULL;
      HZ_Scroll       : Boolean        := False;
      VR_Scroll       : Boolean        := False;
      Scrollbar_Color : String_Pointer := NULL;
   end record;

   -- reads information from file into Listbox,
   -- assumes keyword already read.
   procedure Read_Widget(Widget : in out Listbox;
      File        : in Ada.Text_IO.File_Type);

   procedure Get_Widget_Params(Widget : in out Listbox;
      Params : in String);
      
   -- Writes information to file from Listbox
   procedure Write_Widget(Widget : in Listbox;
      File : in Ada.Text_IO.File_Type);

   -- Generate Ada/Tcl code for this window.
   procedure Generate_Widget(Widget : in Listbox;
      Window_Name : in String;
      File : in Ada.Text_IO.File_Type;
      Tcl_Only : in Boolean);

   -- display the widget to a window
   procedure Display_Widget(Widget : in Listbox;
      Interp      : in Tcl.Tcl_Interp;
      Window_Name : in String);

   procedure Set_Property_Entries(Widget : in out Listbox;
      Pos : in out Auto_Position.Positioner);

   procedure Apply_Properties(Widget : in out Listbox);
   procedure Check_Properties(Widget : in out Listbox;
      Ok : out Boolean);

   function Properties_Dialog_Name(Widget : in Listbox) return String;

end gui.widget.listbox;