---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer
--                                                           
--  GUI-WIDGET-PICTURE.ADS
--  Description : GUI Widget Picture (from GIF)
--                       
--  By: Martin Carlisle 
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
-- 02/14/99 Dmitri Anisimkov : Set_Property_Entries instead Set_Properties
-- for creation inheritable propery editors without designer
-- 03/20/99 Dmitri Anisimkov : Add Bound_Relief property 
---------------------------------------------------------------
with Tcl_Helpers;
with Auto_Position;

package Gui.Widget.Picture is

   type Picture is new GUI_Widget with record
      Image       : String_Pointer := NULL;
      Bound_Relief    : Relief := Flat;
   end record;

   -- reads information from file into Picture,
   -- assumes keyword already read.
   procedure Read_Widget(Widget : in out Picture;
      File        : in Ada.Text_IO.File_Type);

   procedure Get_Widget_Params(Widget : in out Picture;
      Params : in String);

   -- Writes information to file from Picture
   procedure Write_Widget(Widget : in Picture;
      File : in Ada.Text_IO.File_Type);

   -- Generate Ada/Tcl code for this window.
   procedure Generate_Widget(Widget : in Picture;
      Window_Name : in String;
      File : in Ada.Text_IO.File_Type;
      Tcl_Only : in Boolean);

   -- display the widget to a window
   procedure Display_Widget(Widget : in Picture;
      Interp      : in Tcl.Tcl_Interp;
      Window_Name : in String);

   procedure Set_Property_Entries(Widget : in out Picture;
      Pos : in out Auto_Position.Positioner);

   procedure Apply_Properties(Widget : in out Picture);
   procedure Check_Properties(Widget : in out Picture;
      Ok : out Boolean);

   procedure Generate_Context_Clause(Widget : in Picture;
      File : in Ada.Text_IO.File_Type);

   procedure Generate_Action_Binding(Widget : in Picture;
      File : in Ada.Text_IO.File_Type);

   function Properties_Dialog_Name(Widget : in Picture) return String;

end gui.widget.Picture;