---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer
--                                                           
--  GUI-WIDGET-PROGRESS.ADS
--  Description : GUI Widget Progress
--                       
--  By: Dmitri Anisimkov
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
with Auto_Position;
with Tcl_Utilities;

package Gui.Widget.Progress is

   subtype Progress_Value is Tcl_Utilities.Progress_Value;

   type Progress is new Gui_Widget with record
      Entire_Color   : String_Pointer := null;
      Part_Color	   : String_Pointer := null;
      Entire_Relief  : Relief := Sunken;
      Part_Relief	   : Relief := Raised;
      Value          : Progress_Value := 0.0;
   end record;

   -- Get information from string into Progress,
   procedure Get_Widget_Params(Widget : in out Progress;
      Params : in String);

   -- Writes information to file from Progress
   procedure Write_Widget(Widget : in Progress;
      File : in Ada.Text_Io.File_Type);

   -- Generate Ada/Tcl code for this window.
   procedure Generate_Widget(Widget : in Progress;
      Window_Name : in String;
      File : in Ada.Text_Io.File_Type;
      Tcl_Only : in Boolean);

   -- display the widget to a window
   procedure Display_Widget(Widget : in Progress;
      Interp      : in Tcl.Tcl_Interp;
      Window_Name : in String);

   procedure Set_Property_Entries(Widget : in out Progress;
      Pos : in out Auto_Position.Positioner);

   procedure Apply_Properties(Widget : in out Progress);

   procedure Check_Properties(Widget : in out Progress;
      Ok     : out Boolean);

   function Properties_Dialog_Name(Widget : in Progress) return String;

end Gui.Widget.Progress;