---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer
--                                                           
--  GUI-WIDGET-RADIO_BUTTON.ADB
--  Description : GUI Widget Radio Button 
--                       
--  By: Martin Carlisle and Patrick Maes
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
-- 02/14/99 Dmitri Anisimkov : Set_Property_Entries instead Set_Properties
-- for creation inheritable propery editors without designer
---------------------------------------------------------------
with Ada.Text_Io;
with File_Helpers;
with Tcl_Helpers,Tcl_Utilities;
with Gui_Enum;
with Tcl.Ada;
with Auto_Position;--with Radio_Button_Dialog;
with Main;

-- debugging
with Common_Dialogs;

package body Gui.Widget.Radio_Button is

   -- reads information from file into GUI_Widget,
   -- assumes keyword already read.
   procedure Read_Widget(Widget : in out Radio_Button;
         File        : in Ada.Text_Io.File_Type) is
      Word : Word_Type;
      Last : Natural;
   begin -- Read_Widget
      Read_Widget(Gui_Widget(Widget),File);
      File_Helpers.Get_String(File,Word,Last);
      Widget.Text := new String'(Word(1..Last));
      File_Helpers.Get_String(File,Word,Last);
      Widget.Group := new String'(Word(1..Last));
   end Read_Widget;
   
   procedure Get_Widget_Params(Widget : in out Radio_Button;
      Params : in String) is
   begin
      Get_Widget_Params(Gui_Widget(Widget),Params);
      Widget.Text := new String'(Get_Widget_Param(Params,"text"));
      Widget.Group := new String'(Get_Widget_Param(Params,"group"));
   end;

   -- Writes information to file from GUI_Widget

   procedure Write_Widget(Widget : in Radio_Button;
         File : in Ada.Text_Io.File_Type) is

   begin -- Write_Widget
      Gui_Enum.Io.Put(File,Gui_Enum.Radiobutton);
      Write_Widget(Gui_Widget(Widget),File);
      Put_Widget_Param(File,"text", Widget.Text.all);
      Put_Widget_Param(File,"group", Widget.Group.all);
      Ada.Text_IO.New_Line(File);
   end Write_Widget;

   -- Generate Ada/Tcl code for this window.

   procedure Generate_Widget(Widget : in Radio_Button;
         Window_Name : in String;
         File : in Ada.Text_Io.File_Type;
         Tcl_Only : in Boolean) is

   begin -- Generate_Widget
      Tcl_Helpers.Begin_Block(File,Tcl_Only);
      Tcl_Helpers.Put(File,
         "radiobutton " &
         Window_Name & Widget.Name.all &
         " -anchor w -text ",Tcl_Only);
      Tcl_Helpers.Put_Quotation(File,Tcl_Only);
      Tcl_Helpers.Put(File,
         Widget.Text.all,Tcl_Only);
      Tcl_Helpers.Put_Quotation(File,Tcl_Only);
      Tcl_Helpers.Put(File," -variable " &
         Widget.Group.all,Tcl_Only);
      Tcl_Helpers.Put_Line(File," -value " &
         Widget.Name.all(Widget.Name.all'First..Widget.Name.all'Last),Tcl_Only);
      Generate_Widget(Gui_Widget(Widget),
         Window_Name,File,Tcl_Only);
      Tcl_Helpers.End_Block(File,Tcl_Only);
   end Generate_Widget;

   -- display the widget to a window
   procedure Display_Widget(Widget : in Radio_Button;
         Interp      : in Tcl.Tcl_Interp;
         Window_Name : in String) is
   begin
      Tcl.Ada.Tcl_Eval(Interp,
         "radiobutton " &
         Window_Name & "." & Widget.Name.all &
         " -anchor w -text """ &
         Widget.Text.all & '"' & " -variable " & Widget.Group.all &
         " -value " & Widget.Name.all(Widget.Name.all'First+1..Widget.Name.all'Last));
      Display_Widget(Gui_Widget(Widget),Interp,Window_Name);
   exception when others =>
         Common_Dialogs.Ok_Box("Can't display: " &
            Widget.Name.all, Interp);
   end Display_Widget;

   procedure Set_Property_Entries(Widget : in out Radio_Button;
      Pos : in out Auto_Position.Positioner) is
   begin
      Set_Property_Entries(Gui_Widget(Widget),pos);
      Auto_Position.Add_Message(pos,"Radio button properties","LightBlue");
      Auto_Position.Add_Entry(pos,
         Label => "Text",
         Name  => "entry6",
         Value  => Tcl_Utilities.Unescape(Gui.Widget.Unaccess(Widget.Text)));
      Auto_Position.Add_Entry(pos,
         Label   => "Group name",
         Name    => "entry7",
         Value   => Gui.Widget.Unaccess(Widget.Group));
   end Set_Property_Entries;

   procedure Apply_Properties(Widget : in out Radio_Button) is
      Word        : Word_Type;
      Last        : Integer;
      Window_Name : String := Properties_Dialog_Name(Widget);
   begin
      Apply_Properties(Gui_Widget(Widget));
      Tcl_Utilities.Get_Text_Entry(Main.Interp,
         Window_Name => Window_Name,
         Name        => "entry6",
         Result      => Word,
         Last        => Last);
      if Last >= Word'First then
         Widget.Text := new String'(Tcl_Utilities.Fix_Quotes(
               Word(Word'First..Last)));
      end if;
      Tcl_Utilities.Get_Text_Entry(Main.Interp,
         Window_Name => Window_Name,
         Name        => "entry7",
         Result      => Word,
         Last        => Last);
      if Last >= Word'First then
         Widget.Group := new String'(Tcl_Helpers.Undot_Name(Word(Word'First..Last)));
      end if;
   end Apply_Properties;

   procedure Check_Properties(Widget : in out Radio_Button;
         Ok : out Boolean) is
      Window_Name : String := Properties_Dialog_Name(Widget);
   begin
      Check_Properties(Gui_Widget(Widget),Ok);
      if Ok then
         Ok := Widget.Text /= null;
         if not Ok then
            Tcl_Utilities.Highlight_Text_Entry(Main.Interp,
               Window_Name,"entry6");
         end if;
      end if;
      if Ok then
         Ok := Widget.Group /= null;
         if not Ok then
            Tcl_Utilities.Highlight_Text_Entry(Main.Interp,
               Window_Name,"entry7");
         end if;
      end if;
      if not Ok then
         Tcl.Ada.Tcl_Eval(Main.Interp,"bell ");
      end if;
   end Check_Properties;


   -- return name of properties dialog
   function Properties_Dialog_Name(Widget : in Radio_Button) return String is
   begin
      return ".radio_button_dialog";
   end Properties_Dialog_Name;

end Gui.Widget.Radio_Button;