---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer
--                                                           
--  GUI-WIDGET-STATIC.ADS
--  Description : GUI Widget Static Label
--                       
--  By: Martin Carlisle and Jonathan Busch
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
-- 03/20/99 Dmitri Anisimkov : Add Bound_Relief property 
-- 03/20/99 Dmitri Anisimkov : Change Justification property
--                             to enumeration
---------------------------------------------------------------

with Tcl_Helpers;
with Auto_Position;

package Gui.Widget.Static is

   type Justification is (left,center,right);

   type Label is new GUI_Widget with record
      Text      : String_Pointer := NULL;
      FG_Color	  : String_Pointer := NULL;
      BG_Color	  : String_Pointer := NULL;
      Justify   : Justification := left;
      Bound_Relief    : Relief := Flat;
   end record;

   -- reads information from file into Label,
   -- assumes keyword already read.
   procedure Read_Widget(Widget : in out Label;
      File        : in Ada.Text_IO.File_Type);

   procedure Get_Widget_Params(Widget : in out Label;
      Params : in String);

   -- Writes information to file from Label
   procedure Write_Widget(Widget : in Label;
      File : in Ada.Text_IO.File_Type);

   -- Generate Ada/Tcl code for this window.
   procedure Generate_Widget(Widget : in Label;
      Window_Name : in String;
      File : in Ada.Text_IO.File_Type;
      Tcl_Only : in Boolean);

   -- display the widget to a window
   procedure Display_Widget(Widget : in Label;
      Interp      : in Tcl.Tcl_Interp;
      Window_Name : in String);

   procedure Set_Property_Entries(Widget : in out Label;
      Pos : in out Auto_Position.Positioner);

   procedure Apply_Properties(Widget : in out Label);
   procedure Check_Properties(Widget : in out Label;
      Ok : out Boolean);

   function Properties_Dialog_Name(Widget : in Label) return String;

end gui.widget.static;