---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer
--                                                           
--  GUI-WIDGET-TEXT.ADB
--  Description : GUI Widget Text box
--                       
--  By: Martin Carlisle 
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
-- 02/14/99 Dmitri Anisimkov : Set_Property_Entries instead Set_Properties
-- for creation inheritable propery editors without designer
---------------------------------------------------------------
with Gui_Enum;
with File_Helpers;
with Tcl_Helpers,Tcl_Utilities;
with Tcl.Ada;
with Auto_Position;--with TextEntry_Dialog;
with Main;
with Common_Dialogs;

package body Gui.Widget.Text is
   -- reads information from file into Text_Entry,
   -- assumes keyword already read.
   procedure Read_Widget(Widget : in out Text_Entry;
         File : in Ada.Text_IO.File_Type) is

      Word : Word_Type;
      Last : Natural;
   begin -- Read_Widget
      Read_Widget(GUI_Widget(Widget),File);

      File_Helpers.Get_String(File,Word,Last);
      Widget.Enter_Action := new String'(Word(1..Last));
   end Read_Widget;

   procedure Get_Widget_Params(Widget : in out Text_Entry;
      Params : in String) is
   begin
      Get_Widget_Params(Gui_Widget(Widget),Params);
      Widget.Enter_Action := new String'(Get_Widget_Param(Params,"enter"));
   end;

   -- Writes information to file from Text_Entry
   procedure Write_Widget(Widget : in Text_Entry;
         File : in Ada.Text_IO.File_Type) is

   begin -- Write_Widget
      Gui_Enum.IO.Put(File,Gui_Enum.TextEntry);
      Write_Widget(GUI_Widget(Widget),File);
      Put_Widget_Param(File,"enter", Widget.Enter_Action.all);
      Ada.Text_IO.New_Line(File);
   end Write_Widget;

   -- Generate Ada/Tcl code for this window.
   procedure Generate_Widget(Widget : in Text_Entry;
         Window_Name : in String;
         File : in Ada.Text_IO.File_Type;
         Tcl_Only : in Boolean) is

   begin -- Generate_Widget
      Tcl_Helpers.Begin_Block(File,Tcl_Only);
      Tcl_Helpers.Put_Line(File,
         "entry " &
         Window_Name & Widget.Name.all,Tcl_Only);
      Tcl_Helpers.Put(File,
         "bind " &
          Window_Name & Widget.Name.all &
         " <Key-Return> {",Tcl_Only);
      Tcl_Helpers.Put(File,
         "TextEntry" & Tcl_Helpers.Undot_Name(Widget.Name.all),
         Tcl_Only);
      Tcl_Helpers.Put(File,"}",Tcl_Only);
      Tcl_Helpers.New_Line(File,Tcl_Only);
      Generate_Widget(GUI_Widget(Widget),Window_Name,File,Tcl_Only);
      Tcl_Helpers.End_Block(File,Tcl_Only);
   end Generate_Widget;

   procedure Generate_Action(Widget : in Text_Entry;
      File : in Ada.Text_IO.File_Type) is
   begin
      Tcl_Helpers.Generate_Command(File => File,
         Action => Widget.Enter_Action.all,
         Command_Name => "TextEntry" &
            Tcl_Helpers.Undot_Name(Widget.Name.all));
   end Generate_Action;


   procedure Generate_Action_Binding(Widget : in Text_Entry;
      File : in Ada.Text_IO.File_Type) is
   begin
      Tcl_Helpers.Generate_Command_Binding(File => File,
         Command_Name => "TextEntry" &
            Tcl_Helpers.Undot_Name(Widget.Name.all));
   end Generate_Action_Binding;


   procedure Generate_Context_Clause(Widget : in Text_Entry;
      File : in Ada.Text_IO.File_Type) is
   begin
      Tcl_Helpers.Generate_With(File,Widget.Enter_Action.all);
   end Generate_Context_Clause;

   -- display the widget to a window
   procedure Display_Widget(Widget : in Text_Entry;
      Interp      : in Tcl.Tcl_Interp;
      Window_Name : in String) is
   begin
      Tcl.Ada.Tcl_Eval(Interp,
         "entry " &
         Window_Name & "." & Widget.Name.all);
      Display_Widget(Gui_Widget(Widget),Interp,Window_Name);
   exception when others =>
      Common_Dialogs.OK_Box("Can't display: " &
         Widget.Name.all, Interp);
   end Display_Widget;

   procedure Set_Property_Entries(Widget : in out Text_Entry;
      Pos : in out Auto_Position.Positioner) is
   begin
      Set_Property_Entries(Gui_Widget(Widget),pos);
      Auto_Position.Add_Message(Pos,"Text entry properties","LightBlue");
      Auto_Position.Add_Entry(pos,
            Label => "Action on Enter",
            Name   => "entry6",
            Value   => Unaccess(Widget.Enter_Action));
   end Set_Property_Entries;

   procedure Apply_Properties(Widget : in out Text_Entry) is
      Word        : Word_Type;
      Last        : Integer;
      Window_Name : String := Properties_Dialog_Name(Widget);
   begin
      Apply_Properties(Gui_Widget(Widget));
      Tcl_Utilities.Get_Text_Entry(Main.Interp,
         Window_Name => Window_Name,
         Name        => "entry6",
         Result      => Word,
         Last        => Last);
      if Last >= Word'First then
         Widget.Enter_Action := new String'(Word(Word'First..Last));
--         if Widget.Name /= null then
--            Tcl.Ada.Tcl_Eval(Main.Interp,"destroy " &
--               Window_Name);
--         end if;
      end if;
   end Apply_Properties;

   procedure Check_Properties(Widget : in out Text_Entry;
      Ok : out Boolean) is
      Window_Name : String := Properties_Dialog_Name(Widget);
   begin
      Check_Properties(GUI_Widget(Widget),Ok);
      if Ok then
         Ok := Widget.Enter_Action /= null;
         if not Ok then
            Tcl_Utilities.Highlight_Text_Entry(Main.Interp,
               Window_Name,"entry6");
         end if;
      end if;

      if not Ok then
         Tcl.Ada.Tcl_Eval(Main.Interp,"bell ");
      end if;
   end Check_Properties;


   -- return name of properties dialog
   function Properties_Dialog_Name(Widget : in Text_Entry) return String is
   begin
      return ".textentry_dialog";
   end Properties_Dialog_Name;


end Gui.Widget.Text;