---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer
--                                                           
--  GUI-WIDGET-TEXT.ADS
--  Description : GUI Widget Text Box
--                       
--  By: Martin Carlisle 
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
-- 02/14/99 Dmitri Anisimkov : Set_Property_Entries instead Set_Properties
-- for creation inheritable propery editors without designer
---------------------------------------------------------------
with Tcl_Helpers;
with Auto_Position;

package Gui.Widget.Text is

   type Text_Entry is new GUI_Widget with record
      Enter_Action : String_Pointer;
      -- what happens if user presses enter in this entry widget
   end record;

   -- reads information from file into Text_Entry,
   -- assumes keyword already read.
   procedure Read_Widget(Widget : in out Text_Entry;
      File        : in Ada.Text_IO.File_Type);

   procedure Get_Widget_Params(Widget : in out Text_Entry;
      Params : in String);
      
   -- Writes information to file from Text_Entry
   procedure Write_Widget(Widget : in Text_Entry;
      File : in Ada.Text_IO.File_Type);

   -- Generate Ada/Tcl code for this window.
   procedure Generate_Widget(Widget : in Text_Entry;
      Window_Name : in String;
      File : in Ada.Text_IO.File_Type;
      Tcl_Only : in Boolean);

   procedure Generate_Action(Widget : in Text_Entry;
      File : in Ada.Text_IO.File_Type);

   procedure Generate_Action_Binding(Widget : in Text_Entry;
      File : in Ada.Text_IO.File_Type);

   procedure Generate_Context_Clause(Widget : in Text_Entry;
      File : in Ada.Text_IO.File_Type);

   -- display the widget to a window
   procedure Display_Widget(Widget : in Text_Entry;
      Interp      : in Tcl.Tcl_Interp;
      Window_Name : in String);

   procedure Set_Property_Entries(Widget : in out Text_Entry;
      Pos : in out Auto_Position.Positioner);

   procedure Apply_Properties(Widget : in out Text_Entry);
   procedure Check_Properties(Widget : in out Text_Entry;
      Ok : out Boolean);

   function Properties_Dialog_Name(Widget : in Text_Entry) return String;
end Gui.Widget.Text;