---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer
--                                                           
--  GUI.ADB 
--  Description : Save/Load properties utilities
--                       
--  By: Dmitri Anisimkov
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
with
   Gnat.Spitbol,
   Gnat.Spitbol.Patterns,
   Ada.Strings.Fixed,
   Ada.Strings.Maps,
   Ada.Strings.Maps.Constants;
package body Gui is

   use
      Gnat.Spitbol,
      Gnat.Spitbol.Patterns;

   Pre_Pattern : constant Pattern :=
      (Arbno(Notany('"')) &
      Arbno("""" & Arbno(Notany('"')) & """" & Arbno(Notany('"')))) &
      "-";

   function Replace(Source,Pattern,By : String) return String is
      Idx : Natural := Ada.Strings.Fixed.Index(Source,Pattern);
   begin
      if Idx=0 then
         return Source;
      else
         return Source(Source'First..Idx-1)&By&
            Replace(Source(Idx+Pattern'Length..Source'Last),Pattern,By);
      end if;
   end;

   function Get_Widget_Param(Params,Param_Name : String)
         return Boolean is
   begin
      return Match(Params,Pre_Pattern & Param_Name);
   end;

   function Get_Widget_Param(Params,Param_Name : String;
         Default : String := "") return String is
      Result : Vstring;
      Str_Pattern : constant Pattern := Pre_Pattern & Param_Name & Span(' ') & '"' &
         (Arbno(Notany('"')) & Arbno(Arbno("""""") & Arbno(Notany('"'))))
         ** Result & '"' & Notany('"');
   begin
      if Match(Params&' ',  Str_Pattern) then
         return Replace(S(Result),"""""","""");
      else
         return Default;
      end if;
   end;

   function Get_Widget_Param(Params,Param_Name : String;
         Default : Integer := 0) return Integer is
      Result : Vstring;
      Int_Pattern : constant Pattern := Pre_Pattern &
         Param_Name & Span(' ') &
         Span(Ada.Strings.Maps.Constants.Decimal_Digit_Set)
         ** Result;
   begin
      if Match(Params,  Int_Pattern) then
         return Integer'Value(S(Result));
      else
         return Default;
      end if;
   end;

   Numeric_Pattern : constant Pattern := Span("0123456789E.+-");

   function Get_Widget_Param(Params,Param_Name : String;
         Default : Float := 0.0) return Float is
      Result : Vstring;
      Float_Pattern : constant Pattern := Pre_Pattern &
         Param_Name & Span(' ') & Numeric_Pattern ** Result;
   begin
      if Match(Params,  Float_Pattern) then
         return Float'Value(S(Result));
      else
         return Default;
      end if;
   end;

   procedure Put_Widget_Param(File  : in Ada.Text_Io.File_Type;
         Param_Name,Param_Value : String) is
   begin
      Ada.Text_Io.Put(File," -"&Param_Name&" """);
      for I in Param_Value'range loop
         if Param_Value(I)='"' then
            Ada.Text_Io.Put(File,'"');
         end if;
         Ada.Text_Io.Put(File,Param_Value(I));
      end loop;
      Ada.Text_Io.Put(File,'"');
   end;

   procedure Put_Widget_Param(File  : in Ada.Text_Io.File_Type;
         Param_Name : String;Param_Value : Integer) is
   begin
      Ada.Text_Io.Put(File," -"&Param_Name&Integer'Image(Param_Value));
   end;

   procedure Put_Widget_Param(File  : in Ada.Text_Io.File_Type;
         Param_Name : String;Param_Value : Float) is
   begin
      Ada.Text_Io.Put(File," -"&Param_Name&Float'Image(Param_Value));
   end;

   package body Fixed_Param_Io is

      package Param_Io is new Ada.Text_Io.Fixed_Io(Num);

      function To_Numeric(Str : String) return Num is
         Result : Num;
         Last : Positive;
      begin
         Param_Io.Get(Str,Result,Last);
         return Result;
      end;

      function To_String(Numeric : Num) return String is
         Result : String(1..40);
         use Ada.Strings.Maps; 
      begin
         Param_Io.Put(Result,Numeric,Exp=>0,
         Aft=>Param_Io.Default_Aft+7); 
         -- Aft is so big to go around Windows 95 bug when 
         -- Getopenfilename function changes FPU control 
         -- register and after that numeric to string 
         -- convertion with Defaults did not works properly
         return Ada.Strings.Fixed.Trim(Result,
            To_Set(' '),To_Set('0'));
      end;

      function Get_Widget_Param(Params,Param_Name : String) return Num is
         Result_Str : Vstring;
         Num_Pattern : constant Pattern := Pre_Pattern &
            Param_Name & Span(' ') & Numeric_Pattern ** Result_Str;
      begin
         if Match(Params,  Num_Pattern) then
            return To_Numeric(S(Result_Str));
         else
            return Default_Value;
         end if;
      end;

      procedure Put_Widget_Param(File  : in Ada.Text_Io.File_Type;
            Param_Name : String;Param_Value : Num) is
      begin
         Ada.Text_Io.Put(File," -"&Param_Name&' '&To_String(Param_Value));
      end;

   end Fixed_Param_Io;

   procedure Put_Widget_Param(File  : in Ada.Text_Io.File_Type;
         Param_Name : String;Param_Value : Boolean) is
   begin
      if Param_Value then
         Ada.Text_Io.Put(File," -"&Param_Name);
      end if;
   end;

end Gui;