---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer     
--                                                           
--  RAPID.ADB 
--  Description : Main procedure 
--                       
--  By: Martin Carlisle
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
with Main;
with Tcl.Tk;
with Tcl.Ada;
with Tcl_Bindings;
with Toolbar;
with Ada.Exceptions;
with Ada.Command_Line;
with State;
with Gui;
with Gui.Window;
with Limits;
with Tcl_Utilities;
with Common_Dialogs;
--The following with clause is needed only for exception block below.  
--The exception block behaved poorly in practice
with File_Menu;

procedure Rapid is
   pragma Linker_Options("-ltcl80");
   pragma Linker_Options("-ltk80");
   -- Tcl/Tk libraries (for all platforms)
   Filename,Directory : String(1..Limits.Max_Name_Length);
   File_Last,Dir_Last : Natural;
begin
   Main.Generate_Window(Main.Interp);
   Tcl_Bindings.Generate_Bindings(Main.Interp);
   Toolbar.Initialize_Toolbar;
   Tcl.Ada.Tcl_Eval(Main.Interp,"wm protocol . WM_DELETE_WINDOW " &
      "delete_mainwindow_action");
   Tcl_Utilities.Change_Window_Title(Interp => Main.Interp,
      Window_Name => ".",
      Title       => "RAPID");
   Tcl.Ada.Tcl_Eval(Main.Interp,"wm minsize . 300 58");

   -- adapted from submission by Dmitri Anisimkov
   -- allows Windows users to double click a .GUI file
   -- and have it be opened in RAPID

   if Ada.Command_Line.Argument_Count=1 then
      begin
         Tcl.Ada.Tcl_Eval(Main.Interp,
            "set filename [file join [pwd] """ &
            Tcl_Utilities.Fix_Quotes(Ada.Command_Line.Argument(1)) &
            """]" & Ascii.Lf &
            "set initialdir [file dirname $filename]");
         Tcl_Utilities.Get_Value(Interp => Main.Interp,
            Name   => "initialdir",
            Result => Directory,
            Last   => Dir_Last);
         Tcl_Utilities.Get_Value(Interp => Main.Interp,
            Name   => "filename",
            Result => Filename,
            Last   => File_Last);
         Tcl.Ada.Tcl_Eval(Main.Interp,
            "cd $initialdir");

         State.Set_Current_Window(State.Allocate_Window);
         State.Set_Filename(Filename(Filename'First..File_Last));
         State.Set_Directory(Directory(Directory'First..Dir_Last));

         Gui.Window.Read_Window(Window => State.Get_Current_Window.all,
            Filename => State.Get_Filename);
         Gui.Window.Display_Window(Interp => Main.Interp,
            Window => State.Get_Current_Window.all);
         State.Set_Changed(False);
      exception
         when others =>
            Common_Dialogs.Ok_Box("Unable to open " &
               Filename(Filename'First..File_Last),Main.Interp);
      end;
   end if;


   Tcl.Tk.Tk_Mainloop;
      exception when E:others =>
         Common_Dialogs.OK_Box("exception caught -- contact author:" &
            Ada.Exceptions.Exception_Information(E),
            Main.Interp);
         File_Menu.Exit_Choice;
         raise;
end Rapid;