---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer     
--                                                           
--  STATE.ADB
--  Description : Things to keep track of
--                       
--  By: Martin Carlisle
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
with Ada.Unchecked_Deallocation;
with Gui,Gui.Window;

package body state is
   Changed_Since_Save : Boolean := False;
   Dialog_Up          : Boolean := False;
   Filename           : String(1..255) := "Untitled" & string'(9..255 => ' ');
   Filename_Last      : Natural := 8;
   Directory          : String(1..255);
   Directory_Last     : Natural;
   Current_Window     : Gui.Window.Window_Pointer := NULL;
   Current_Selection  : Gui.Widget.Widget_Access := NULL;

   function Allocate_Window return Gui.Window.Window_Pointer is

      tempvar : Gui.Window.Window_Pointer;

   begin -- Allocate_Window
      tempvar := new Gui.Window.GUI_Window;
      return tempvar;
   end Allocate_Window;


   procedure Destroy_Window(Window : in out Gui.Window.Window_Pointer) is
      procedure Destroy is new Ada.Unchecked_Deallocation(Gui.Window.GUI_Window,
         Gui.Window.Window_Pointer);
      procedure Destroy is new Ada.Unchecked_Deallocation(String,
         Gui.String_Pointer);
   begin -- Destroy_Window
      Destroy(Window.Window_Name);
      Destroy(Window.Filename);
      -- really should deallocate all of files/widgets here
      Destroy(Window);
   end Destroy_Window;

   function Get_Changed return Boolean is
   begin
      return Changed_Since_Save;
   end Get_Changed;

   function Dialog_Running return Boolean is
   begin
      return Dialog_Up;
   end Dialog_Running;

   function Get_Filename return String is
   begin
      return Filename(1..Filename_Last);
   end Get_Filename;

   function Get_Directory return String is
   begin
      return Directory(1..Directory_Last);
   end Get_Directory;

   procedure Set_Filename(Name : in String) is
   begin
      Filename(1..Name'Length) := Name;
      Filename_Last := Name'Length;
   end Set_Filename;

   procedure Set_Directory(Name : in String) is
   begin
      Directory(1..Name'Length) := Name;
      Directory_Last := Name'Length;
   end Set_Directory;

   procedure Set_Changed(Changed : in Boolean) is
   begin
      Changed_Since_Save := Changed;
   end Set_Changed;

   procedure Set_Dialog_Running(Running : in Boolean) is
   begin
      Dialog_Up := Running;
   end Set_Dialog_Running;

   function Get_Current_Window return Gui.Window.Window_Pointer is
   begin
      return Current_Window;
   end Get_Current_Window;

   procedure Set_Current_Window(Window : Gui.Window.Window_Pointer) is
   begin
      Current_Window := Window;
      Current_Selection := NULL;
   end Set_Current_Window;

   function Get_Selection return Gui.Widget.Widget_Access is
   begin
      return Current_Selection;
   end Get_Selection;

   procedure Set_Selection(Selection : in Gui.Widget.Widget_Access) is
   begin
      Current_Selection := Selection;
   end Set_Selection;

end state;