---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer     
--                                                           
--  SUBWINDOW_ACTIONS.ADB 
--  Description : Those actions from events in a subwindow
--                       
--  By: Martin Carlisle
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
-- Dmitri Anisimkov 3/11/99 Resize widget with windows functionality.
---------------------------------------------------------------
with Main;
with Common_Dialogs;
with Gui.Window;
with State;
with Toolbar;
with Tcl.Ada;
with Gui;
with Gui.Widget;
use type Gui.String_Pointer,Gui.Widget.Widget_Access;
use type Gui.Window.Window_Pointer;
with Menu_Edit;
with Ada.Unchecked_Deallocation;
with Gui_Enum;

package body Subwindow_Actions is
   -- keep this around as global state for Done_Add_Widget callback
   -- and also for Done_Modify_Widget callback
   The_Widget  : Gui.Widget.Widget_Access;
   type Action is (Add,Modify);
   Move_Corner_X, Move_Corner_Y : Integer;
   Current_Action : Action;
   Fake_Name : Gui.String_Pointer; -- used to change name before verifying

   procedure Resize(Name : in String;
         Width : in Integer;
         Height : in Integer) is
      Window : Gui.Window.Window_Pointer := State.Get_Current_Window;
   begin -- Resize
      if Gui.Window.Make_Widget_Area_Name(Window.Window_Name.all) = Name then
         if Width /= Window.Width or else Height /= Window.Height then
            Gui.Window.Resize_Window(Interp => Main.Interp,
               Window => Window.all,
               Width  => Width,
               Height => Height);
            State.Set_Changed(True);
         end if;
      end if;
   end Resize;

   function Min(L,R : in Integer) return Integer is
   begin
      if L<R then return L; else return R; end if;
   end Min;

   function Max(L,R : in Integer) return Integer is
   begin
      if L>R then return L; else return R; end if;
   end Max;

   procedure Add_Widget(Startx : in Integer; Starty : in Integer;
         Endx : in Integer; Endy : in Integer) is
      Widget_Type : Toolbar.Widget_Names := Toolbar.Get_Selected_Widget;
      Smallx : Integer := Min(Startx,Endx);
      Smally : Integer := Min(Starty,Endy);
      Bigx   : Integer := Max(Startx,Endx);
      Bigy   : Integer := Max(Starty,Endy);
      Width  : Integer := Bigx-Smallx;
      Height : Integer := Bigy-Smally;
   begin
      -- don't add if dialog already running
      if State.Dialog_Running then
         return;
      end if;

      Current_Action := Add;
      if Width > 0 and then Height > 0 then
         The_Widget := Gui_Enum.Allocate_Widget(Widget_Type);
         Gui.Widget.Set_Location(The_Widget,
            Smallx, Smally, Width, Height);
         State.Set_Dialog_Running(True);
         Gui.Widget.Set_Properties(The_Widget.all);
      end if;
   end Add_Widget;

   -- really this is called from Edit_Menu, and probably doesn't
   -- belong here, but it was easier to reuse code from Add_Widget
   procedure Duplicate(Widget : in Gui.Widget.Widget_Access) is
   begin
      Current_Action := Add;
      Widget.Name    := null;
      The_Widget := Widget; -- global variable to determine who
      -- is active right now
      Gui.Widget.Set_Properties(Widget.all);
   end Duplicate;

   function Find_Widget(Window : in Gui.Window.Window_Pointer;
         Name : in String) return Gui.Widget.Widget_Access is
      Real_Name_Start : Integer;
   begin
      -- note we only have stored after the last . in the widget
      Real_Name_Start := Name'Last+1;
      while Real_Name_Start > Name'First and then
            Name(Real_Name_Start-1) /= '.' loop
         Real_Name_Start := Real_Name_Start - 1;
      end loop;
      return
         Gui.Widget.Widget_List_Package.Retrieve(
         Window.Widget_List,
         Gui.Widget.Widget_Key_List_Package.Find(
            Key => Name(Real_Name_Start..Name'Last),
            Ptr => Window.Widget_List));
   exception when Gui.Widget.Widget_Key_List_Package.Not_Found =>
         return null;
   end Find_Widget;

   procedure Done_Add_Widget(Dismiss : in Boolean := True) is
      Window      : Gui.Window.Window_Pointer := State.Get_Current_Window;
      Display_Name : String := Gui.Window.Make_Widget_Area_Name(
         Window.Window_Name.all);
      Ok : Boolean;
      Temp_Name : Gui.String_Pointer;
      procedure Free is new Ada.Unchecked_Deallocation(String,
         Gui.String_Pointer);
   begin
      Gui.Widget.Apply_Properties(The_Widget.all);

      -- this is really grotesque, but we need to make sure the
      -- name isn't duplicated.  Swap in fake name and do a find.
      -- if found, then bad.  Then swap back in real name
      Temp_Name := The_Widget.Name;
      The_Widget.Name := Fake_Name;

      if Temp_Name /= null and then
            Find_Widget(Window,Temp_Name.all) /= null then
         Common_Dialogs.Ok_Box(Temp_Name.all & " is already in use." &
            Ascii.Lf & "Pick another name",Main.Interp);
         Free(Temp_Name);
         The_Widget.Name := null;
      else
         The_Widget.Name := Temp_Name;
      end if;

      Gui.Widget.Check_Properties(The_Widget.all,Ok);
      if not Ok then
         return;
      end if;


      if Dismiss then
         Gui.Widget.Close_Properties(The_Widget.all);
         State.Set_Dialog_Running(False);
      else
         Current_Action := Modify;
      end if;

      -- add to Window.Widget_List
      Gui.Widget.Widget_List_Package.Addtorear(L => Window.Widget_List,
         X => The_Widget);
      Gui.Widget.Display_Widget(Widget => The_Widget.all,
         Window_Name => Display_Name,
         Interp      => Main.Interp);
      Tcl.Ada.Tcl_Eval(Main.Interp,"update");
      State.Set_Changed(True);
   end Done_Add_Widget;

   procedure Modify_Widget(Name : in String) is
      Window     : Gui.Window.Window_Pointer := State.Get_Current_Window;
   begin
      -- don't modify if dialog already running
      if State.Dialog_Running then
         return;
      end if;

      Current_Action := Modify;
      The_Widget := Find_Widget(Window,Name);
      if The_Widget /= null then
         State.Set_Dialog_Running(True);
         Gui.Widget.Set_Properties(The_Widget.all);
      end if;
   end Modify_Widget;

   procedure Cancel_Properties_Dialog is
      Ok : Boolean;
   begin
      if Current_Action = Modify then
         Gui.Widget.Check_Properties(The_Widget.all,Ok);
         if Ok then
            Gui.Widget.Close_Properties(The_Widget.all);
         end if;
      else -- Add
         Gui.Widget.Close_Properties(The_Widget.all);
      end if;
   end Cancel_Properties_Dialog;

   procedure Done_Modify_Widget(Dismiss : in Boolean := True) is
      Window      : Gui.Window.Window_Pointer := State.Get_Current_Window;
      Display_Name : String := Gui.Window.Make_Widget_Area_Name(
         Window.Window_Name.all);
      Selection  : Gui.Widget.Widget_Access := State.Get_Selection;
      Ok : Boolean;
      Temp_Name : Gui.String_Pointer;
      procedure Free is new Ada.Unchecked_Deallocation(String,
         Gui.String_Pointer);
   begin
      Gui.Widget.Apply_Properties(The_Widget.all);

      -- this is really grotesque, but we need to make sure the
      -- name isn't duplicated.  Swap in fake name and do a find.
      -- if found, then bad.  Then swap back in real name
      Temp_Name := The_Widget.Name;
      The_Widget.Name := Fake_Name;

      if Temp_Name /= null and then
            Find_Widget(Window,Temp_Name.all) /= null then
         Common_Dialogs.Ok_Box(Temp_Name.all & " is already in use." &
            Ascii.Lf & "Pick another name",Main.Interp);
         Free(Temp_Name);
         The_Widget.Name := null;
      else
         The_Widget.Name := Temp_Name;
      end if;

      Gui.Widget.Check_Properties(The_Widget.all,Ok);
      if not Ok then
         return;
      end if;

      if Dismiss then
         Gui.Widget.Close_Properties(The_Widget.all);
         State.Set_Dialog_Running(False);
      end if;
      Gui.Widget.Undisplay_Widget(Widget => The_Widget.all,
         Window_Name => Display_Name,
         Interp      => Main.Interp);
      Gui.Widget.Display_Widget(Widget => The_Widget.all,
         Window_Name => Display_Name,
         Interp      => Main.Interp);
      if Selection = The_Widget then
         Gui.Widget.Move_Highlight(Widget => The_Widget.all,
            Interp      => Main.Interp,
            Window_Name => Gui.Window.Make_Widget_Area_Name(
               Window.Window_Name.all));
      end if;
      Tcl.Ada.Tcl_Eval(Main.Interp,"update");
      State.Set_Changed(True);
   end Done_Modify_Widget;

   procedure Done_Properties_Dialog is
   begin
      if Current_Action = Add then
         Done_Add_Widget;
      elsif Current_Action = Modify then
         Done_Modify_Widget;
      end if;
   end Done_Properties_Dialog;

   procedure Apply_Properties_Dialog is
   begin
      if Current_Action = Add then
         Done_Add_Widget(Dismiss => False);
      elsif Current_Action = Modify then
         Done_Modify_Widget(Dismiss => False);
      end if;
   end Apply_Properties_Dialog;

   procedure Select_Widget(Name : in String) is
      Window     : Gui.Window.Window_Pointer := State.Get_Current_Window;
      Selection  : Gui.Widget.Widget_Access := State.Get_Selection;
   begin
      -- don't select if dialog already running
      if State.Dialog_Running then
         return;
      end if;

      if Selection /= null then
         Gui.Widget.Unhighlight(Widget => Selection.all,
            Interp      => Main.Interp,
            Window_Name => Gui.Window.Make_Widget_Area_Name(
               Window.Window_Name.all));
      end if;

      The_Widget := Find_Widget(Window,Name);
      if The_Widget /= null then
         Move_Corner_X := The_Widget.X;
         Move_Corner_Y := The_Widget.Y;
         Gui.Widget.Highlight(Widget => The_Widget.all,
            Interp      => Main.Interp,
            Window_Name => Gui.Window.Make_Widget_Area_Name(
               Window.Window_Name.all));
      end if;

      State.Set_Selection(The_Widget);
   end Select_Widget;

   procedure Edit_Menu is
      Window      : Gui.Window.Window_Pointer := State.Get_Current_Window;
   begin
      -- don't edit if dialog already running
      if State.Dialog_Running then
         return;
      end if;

      if Window /= null then
         declare
            Display_Name : String := Gui.Window.Make_Display_Name(
               Window.Window_Name.all);
         begin
            Menu_Edit.Edit(Main.Interp,Window.Menu,Display_Name);
         end;
      end if;
   end Edit_Menu;


   procedure Move_Widget(Name : in String; Startx : in Integer;
         Starty : in Integer;
         Endx : in Integer; Endy : in Integer) is
      Window      : Gui.Window.Window_Pointer := State.Get_Current_Window;
   begin
      -- don't move if dialog already running
      if State.Dialog_Running then
         return;
      end if;

      The_Widget := Find_Widget(Window,Name);
      if The_Widget /= null then
         -- this is really peculiar code, but
         -- start and end are relative to the upper left corner
         -- of the widget, which moves as we go
         The_Widget.X      := The_Widget.X + Endx-Startx;
         The_Widget.Y      := The_Widget.Y + Endy-Starty;
      end if;
      Tcl.Ada.Tcl_Eval(Main.Interp,"place configure " & Name &
         Gui.Widget.Place_Coords(The_Widget.all));
      Gui.Widget.Move_Highlight(Widget => The_Widget.all,
         Interp      => Main.Interp,
         Window_Name => Gui.Window.Make_Widget_Area_Name(
            Window.Window_Name.all));
      State.Set_Changed(True);
   end Move_Widget;

   procedure Resize_Widget(Handle : in String;
         Name : in String;
         Startx : in Integer; Starty : in Integer;
         Endx   : in Integer; Endy   : in Integer) is
      Window      : Gui.Window.Window_Pointer := State.Get_Current_Window;
   begin
      -- don't move if dialog already running
      if State.Dialog_Running then
         return;
      end if;

      The_Widget := Find_Widget(Window,Name);
      if The_Widget /= null then
         -- this is really peculiar code, but
         -- start and end are relative to the upper left corner
         -- of the handle, which moves as we go
         if Handle = "nw" then
            The_Widget.X      := The_Widget.X + Endx-Startx;
            The_Widget.Y      := The_Widget.Y + Endy-Starty;
            The_Widget.Width  := The_Widget.Width  + Startx-Endx;
            The_Widget.Height := The_Widget.Height + Starty-Endy;
         elsif Handle = "ne" then
            The_Widget.Y      := The_Widget.Y + Endy-Starty;
            The_Widget.Width  := The_Widget.Width  + Endx-Startx;
            The_Widget.Height := The_Widget.Height + Starty-Endy;
         elsif Handle = "sw" then
            The_Widget.X      := The_Widget.X + Endx-Startx;
            The_Widget.Width  := The_Widget.Width  + Startx-Endx;
            The_Widget.Height := The_Widget.Height + Endy-Starty;
         elsif Handle = "se" then
            The_Widget.Width  := The_Widget.Width  + Endx-Startx;
            The_Widget.Height := The_Widget.Height + Endy-Starty;
         elsif Handle = "nn" then
            The_Widget.Y      := The_Widget.Y + Endy-Starty;
            The_Widget.Height := The_Widget.Height + Starty-Endy;
         elsif Handle = "ww" then
            The_Widget.X      := The_Widget.X + Endx-Startx;
            The_Widget.Width  := The_Widget.Width  + Startx-Endx;
         elsif Handle = "ss" then
            The_Widget.Height := The_Widget.Height + Endy-Starty;
         elsif Handle = "ee" then
            The_Widget.Width  := The_Widget.Width  + Endx-Startx;
         end if;
      end if;
      Tcl.Ada.Tcl_Eval(Main.Interp,"place configure " & Name &
         Gui.Widget.Place_Coords(The_Widget.all));
      Gui.Widget.Move_Highlight(Widget => The_Widget.all,
         Interp      => Main.Interp,
         Window_Name => Gui.Window.Make_Widget_Area_Name(
            Window.Window_Name.all));
      State.Set_Changed(True);
   end Resize_Widget;

   procedure Resize_Window(
         Startx : in Integer; Starty : in Integer;
         Endx   : in Integer; Endy   : in Integer) is
      Window      : Gui.Window.Window_Pointer := State.Get_Current_Window;
   begin
      Gui.Window.Resize_Window(Main.Interp,
         Window.all,
         Startx, Starty, Endx, Endy);
      State.Set_Changed(True);
   end Resize_Window;

begin
   Fake_Name := new String'("dontuseme" & Character'Last);
end Subwindow_Actions;