---------------------------------------------------------------
--                                                           
--  TCL_HELPERS 
--  Description : Use to auto generate either Tcl or Ada (TASH) code
--                       
--  By: Martin Carlisle 
--  Additions by: Jonathan Busch
--                                         
-- TCL_HELPERS is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- TCL_HELPERS is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
with Ada.Text_IO;

package Tcl_Helpers is
   -- begin a connected blob of Tcl code.
   procedure Begin_Block(File : Ada.Text_IO.File_Type;
      Tcl_Only : in Boolean := False);

   -- put a line inside a connected blob
   procedure Put(File : Ada.Text_IO.File_Type;
      Item     : in String;
      Tcl_Only : in Boolean := False);

   -- put a line inside a connected blob
   -- add a new_line to the Ada code 
   -- use if worried about line length
   procedure Continuation_Put(File : Ada.Text_IO.File_Type;
      Item     : in String;
      Tcl_Only : in Boolean := False);

   -- put a new line inside a connected blob
   procedure New_Line(File : Ada.Text_IO.File_Type;
      Tcl_Only : in Boolean := False);

   -- put a line inside a connected blob
   -- equivalent to put followed by new_line
   procedure Put_Line(File : Ada.Text_IO.File_Type;
      Item     : in String;
      Tcl_Only : in Boolean := False);

   -- writes a quotation mark to the file
   procedure Put_Quotation(File : Ada.Text_IO.File_Type;
      Tcl_Only : in Boolean := False);

   -- End a connected blob of Tcl/Tk code      
   procedure End_Block(File : Ada.Text_IO.File_Type;
      Tcl_Only : in Boolean := False);

   -- generate the command procedure
   procedure Generate_Command(File : Ada.Text_IO.File_Type;
         Command_Name     : in String;
         Action           : in String;
         Number_Arguments : in Natural := 0;
         Client_Data : in String := "");

   -- generate the binding to the command
   procedure Generate_Command_Binding(File : Ada.Text_IO.File_Type;
         Command_Name : in String;
         Client_Data_Need : boolean := false);

   -- generate context clause needed for action
   procedure Generate_With(File : in Ada.Text_IO.File_Type;
         Action     : in String);

   -- return name with "." converted to "_"
   function Undot_Name(Name : in String) return String;

   -- Returns string containing Write string for colors
   function Colors(FG : in String;
                   BG : in String) return String;

end Tcl_Helpers;