---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer     
--                                                           
--  TOOLBAR.ADB 
--  Description : Manages the widget toolbar
--                       
--  By: Martin Carlisle
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
-- 02/04/99 (mcc) : Implement suggestion by Dimitry Anisimkov
--                  to simplify addition of widgets to the toolbar
---------------------------------------------------------------

with Main;
with Tcl.Ada;
package body Toolbar is
   Current_Widget_Type : Widget_Names := Textbutton;

   type String_Access is access constant String;

   Textbutton_Str    : aliased constant String := ".textBButton";
   Picturebutton_Str : aliased constant String := ".pictureBButton";
   Label_Str         : aliased constant String := ".labelButton";
   Textentry_Str     : aliased constant String := ".textEntryButton";
   Checkbutton_Str   : aliased constant String := ".check_button";
   Radiobutton_Str   : aliased constant String := ".radio_button";
   Picture_Str       : aliased constant String := ".picture_select";
   Listbox_Str       : aliased constant String := ".listboxButton";
   Progress_Str      : aliased constant String := ".progress";

   Tool_Button_Widget_Names : array(Widget_Names) of String_Access := (
      Textbutton_Str'access,
      Picturebutton_Str'access,
      Label_Str'access,
      Textentry_Str'access,
      Checkbutton_Str'access,
      Radiobutton_Str'access,
      Picture_Str'access,
      Listbox_Str'access,
      Progress_Str'access
      );

   ----------------------------------------------------------------
   -- procedure Select_Widget
   --
   -- undepress old Current_Widget, then depress new Current_Widget
   -- update value of Current_Widget
   ----------------------------------------------------------------
   procedure Select_Widget(Widget_Type : Widget_Names) is
   begin -- Select_Widget
      Tcl.Ada.Tcl_Eval(Main.Interp,
         Tool_Button_Widget_Names(Current_Widget_Type).all
         &" configure -relief raised");

      Tcl.Ada.Tcl_Eval(Main.Interp,
         Tool_Button_Widget_Names(Widget_Type).all
         &" configure -relief sunken");

      Current_Widget_Type := Widget_Type;
   end Select_Widget;


   function Get_Selected_Widget return Widget_Names is
   begin -- Get_Selected_Widget
      return Current_Widget_Type;
   end Get_Selected_Widget;

   procedure Initialize_Toolbar is
   begin
      Current_Widget_Type := Label;
      Tcl.Ada.Tcl_Eval(Main.Interp,
         Tool_Button_Widget_Names(Current_Widget_Type).all &
         " configure -relief sunken");
   end Initialize_Toolbar;

end Toolbar;